<?php

require ('config.php');

define ("CONFIG_DOC", "config.xml");

function getPINsafeAgent() {
	$config = new Config();
	$config->load(CONFIG_DOC);
	return new PINsafe($config->server, $config->port, $config->context, $config->secret, $config->ssl);
}

class PINsafe {
	// PINsafe server
	private $server;

	// PINsafe port
	private $port;

	// PINsafe context
	private $context;

	// PINsafe agent secret
	private $secret;

	// SSL flag
	private $ssl;

	private $responseCode;
	
	private $warning;

	public function __construct($server, $port, $context, $secret, $ssl) {
		$this->server = $server;
		$this->port = $port;
		$this->context = $context;
		$this->secret = $secret;
		$this->ssl = $ssl;
	}

	private function getProtocol() {
		if ($this->ssl) {
			return "https://";
		}
		return "http://";
	}

	private function getUrl() {
		return $this->getProtocol() . "$this->server:$this->port/$this->context";
	}

	private function getAgentUrl() {
		return $this->getUrl() . '/AgentXML';
	}

	private function getImageUrl() {
		return $this->getUrl() . '/SCImage';
	}

	private function newRequest($type) {
		$doc = new DOMDocument('1.0', 'UTF-8');
		$elem = $doc->createElement('SASRequest');
		$doc->appendChild($elem);
		$this->addParameter($doc, 'Action', $type);
		$this->addParameter($doc, 'Secret', $this->secret);
		$this->addParameter($doc, 'Version', '1.0');
		return $doc;
	}

	private function addParameter($doc, $name, $value) {
	  $elem = $doc->createElement($name, $value);
	  $doc->documentElement->appendChild($elem);
  }

	private function sendRequest($doc) {
		$req = new HttpRequest($this->getAgentUrl(), HTTP_METH_POST);
		$req->setContentType('text/xml');
		$req->addHeaders(array('Content-Encoding'=>'UTF-8'));
		$req->setBody($doc->saveXML());
		$req->send();
		$this->responseCode = $req->getResponseCode();
		if ($this->responseCode == 200) {
			return $req->getResponseBody();
		}
		return NULL;
	}

	public function getResponseCode() {
		return $this->responseCode;
	}
	
	public function getWarning() {
		return $this->warning;
	}

	public function startSession($username) {
		$doc = $this->newRequest('sessionstart');
		$this->addParameter($doc, 'username', $username);
		$response = $this->sendRequest($doc);
		if ($response) {
			$respDoc = DOMDocument::loadXML($response);
			$sessionIDs = $respDoc->getElementsByTagName('SessionID');
			if ($sessionIDs->length > 0) {
				return $sessionIDs->item(0)->textContent;
			}
		}
		return 0;
	}

	private function sendRequestAndTest($doc) {
		$response = $this->sendRequest($doc);
		if ($response) {
			$respDoc = DOMDocument::loadXML($response);
			$warnings = $respDoc->getElementsByTagName('Warning');
			$this->warning = '';
			if ($warnings->length > 0) {
				$this->warning = $warnings->item(0)->textContent;
			}
			$results = $respDoc->getElementsByTagName('Result');
			if ($results->length > 0) {
				return ($results->item(0)->textContent == 'PASS');
			}
		}
		return false;
	}

	public function login($username, $password, $otc) {
		$doc = $this->newRequest('login');
		$this->addParameter($doc, 'Username', $username);
		$this->addParameter($doc, 'Password', $password);
		$this->addParameter($doc, 'OTC', $otc);
		return $this->sendRequestAndTest($doc);
	}

	public function changePIN($username, $oldPassword, $oldOTC, $newPassword, $newOTC) {
		$doc = $this->newRequest('changepin');
		$this->addParameter($doc, 'Username', $username);
		$this->addParameter($doc, 'Password', $oldPassword);
		$this->addParameter($doc, 'NewPassword', $newPassword);
		$this->addParameter($doc, 'OTC', $oldOTC);
		$this->addParameter($doc, 'NewOTC', $newOTC);
		return $this->sendRequestAndTest($doc);
	}

	public function reset($username, $resetCode) {
		$doc = $this->newRequest('reset');
		$this->addParameter($doc, 'Username', $username);
		$this->addParameter($doc, 'ResetCode', $resetCode);
		return $this->sendRequestAndTest($doc);
	}

	public function requestResetCode($username) {
		$doc = $this->newRequest('resetcode');
		$this->addParameter($doc, "Username", $username);
		return $this->sendRequestAndTest($doc);
	}

	public function getImageForSession($sessionID) {
		$req = new HttpRequest($this->getImageUrl() . "?sessionid=$sessionID");
		$req->send();
		return $req->getResponseBody();
	}

	public function getImageForUser($userID) {
		$req = new HttpRequest($this->getImageUrl() . "?userid=$userID");
		$req->send();
		return $req->getResponseBody();
	}
}

?>